
# MCaFE-e1 (Memory Control and Feature Enhancement - economical type 1)

MCaFE-e1はCPLD(ATF1502)を使用したファミコン用のMMCです。  
MCaFE-e1 is an MMC for NES using CPLD (ATF1502).

## ディレクトリ構成/Directories

|ディレクトリ/Directory|内容/Contents|
|---|---|
|anago_script|kazzo/anago用スクリプト<br>script for kazzo/anago.|
|cupl|ファームウェアおよびソースコード<br>Firmware and source code.|
|eagle|プリント基板のCADデータとガーバーデータ<br>Printed circuit board CAD data and Gerber data.|
|fceux|FCEUXに追加するマッパーのソースコード<br>Mapper source code to add to FCEUX.|
|mcafe_e1_demo|機能テスト兼デモプログラム、およびソースコード<br>Functional test and demo program, and source code.|

## 仕様/Specifications

### 概要/Summary

| | |
|---|---|
|PRG ROM capacity|128K|
|PRG ROM window|8K + 8K + 16K fixed|
|PRG RAM capacity|8K|
|PRG RAM window|8K|
|CHR capacity|128K|
|CHR window|4K + 4K|
|Nametable mirroring|H or V, switchable|
|Bus conflicts|No|
|IRQ|Yes|
|Audio|No|

### バンク/Banks

| | |
|---|---|
|CPU $6000-$7FFF|8KB PRG RAM bank|
|CPU $8000-$9FFF|8KB switchable PRG ROM bank|
|CPU $A000-$BFFF|8KB switchable PRG ROM bank|
|CPU $C000-$DFFF|8KB PRG ROM bank, fixed to the second-last bank|
|CPU $E000-$FFFF|8KB PRG ROM bank, fixed to the last bank|
|PPU $0000-$0FFF|4KB switchable CHR bank|
|PPU $1000-$1FFF|4KB switchable CHR bank|

### レジスタ/Registers

#### CHR Bank 0 ($8000-$8FFF)

```
7  bit  0
---D DDDD
   | ||||
   +-++++- New bank value for 4KB CHR Bank at PPU $0000-$0FFF.
           PPU $0000-$0FFFの範囲の4KB CHRバンクに設定する新しい値
```

#### CHR Bank 1 ($9000-$9FFF)

```
7  bit  0
---D DDDD
   | ||||
   +-++++- New bank value for 4KB CHR Bank at CPU $1000-$1FFF.
           PPU $1000-$1FFFの範囲の4KB CHRバンクに設定する新しい値
```

#### PRG Bank 0 ($A000-$AFFF)

```
7  bit  0
---- DDDD
     ||||
     ++++- New bank value for 8KB PRG Bank at CPU $8000-$9FFF.
           CPU $8000-$9FFFの範囲の8KB PRGバンクに設定する新しい値
```

#### PRG Bank 1 ($B000-$BFFF)

```
7  bit  0
---- DDDD
     ||||
     ++++- New bank value for 8KB PRG Bank at CPU $A000-$BFFF.
           CPU $A000-$BFFFの範囲の8KB PRGバンクに設定する新しい値
```

#### IRQ Counter ($E000-$EFFF)

```
7  bit  0
DDDD DDDD
|||| ||||
++++-++++- New IRQ counter value.
           IRQ is a raster interrupt based on PPU A12 with the same mechanism as MMC3.
           Assign BG to pattern table 0 and OBJ to pattern table 1 to use.
           If the value to write is non-zero, IRQ enable, and  cancel the IRQ trigger.
           If the value to write is zero, IRQ disable, and  cancel the IRQ trigger.
           When the counter value is non-zero, the counter value decreases with each the Horizontal Blank.
           When IRQ is enable and  the value of the counter reaches zero, the IRQ is triggered.
           新しいIRQカウンター値。
           IRQは、MMC3と同じ仕組みのPPU A12に基づくラスター割り込みです。
           BGをパターンテーブル0に割り当て、OBJをパターンテーブル1に割り当てて使用します。
           書き込む値がゼロ以外の場合、IRQを有効にして、IRQトリガーをキャンセルします。
           書き込む値がゼロの場合、IRQを無効にして、IRQトリガーをキャンセルします。
           カウンタ値がゼロ以外の場合、カウンタ値は水平ブランクごとに減少します。
           IRQが有効で、カウンターの値がゼロになると、IRQがトリガーされます。
```

#### Mode ($F000-$FFFF)

```
7  bit  0
---- ---M
        |
        +- Nametable mirroring (0: vertical; 1: horizontal)
           ネームテーブルミラーリング (0:垂直 1:水平)
```

## マッパー番号/Mapper number

正式なマッパー番号が割り当てられていないため、
プライベート使用に予約されている領域の4064を使用しています。  
It is using 4064 in the area reserved for private use because 
no official mapper number has been assigned.

kazzo/anago用のスクリプトでは、kazzoがマッパー番号を8bit分しか認識しないため、224を使用しています。  
The script for kazzo/anago uses 224 because kazzo only recognizes the mapper number for 8 bits.

## FCEUXへのマッパーの追加方法/How to add a mapper to FCEUX

mcafe-e1.cppをFCEUXのソースコードの下記ディレクトリに配置します。  
Place mcafe-e1.cpp in the following directory of the FCEUX source code.

> src/boards

Visual studioでFCEUXのプロジェクトを開きます。  
ソリューションエクスプローラーでboardsのディレクトリを選択した後、 [プロジェクト][既存の項目の追加]メニューを選択します。  
ファイルダイアログから、mcafe-e1.cppを選択して プロジェクトに追加します。  
Open the FCEUX project in Visual studio.  
Select the boards directory in Solution Explorer, then select [Project]-[Add Existing Item] menu.  
From the File dialog, select mcafe-e1.cpp to add it to your project.   

次に、src/ines.cpp ファイルをエディタで開いて、追加するマッパーの初期化関数を登録します。  
ines.cppの452行目付近にines番号とマッパーの初期化関数を紐づける定義があります。  
Then open the src/ines.cpp file in an editor and register the mapper initialization function you want to add.  
There is a definition that associates the ines number with the mapper initialization function near line 452 of ines.cpp.

```
static BMAPPINGLocal bmap[] = {
	{"NROM",				  0, NROM_Init},
	{"MMC1",				  1, Mapper1_Init},
	{"UNROM",				  2, UNROM_Init},
(略)

	{"",					0, NULL}
};
```

このリストにMCaFE-e1のマッパーの定義を記述します。  
Describe the mapper definition of MCaFE-e1 in this list. 

```
	{"MCaFE-e1",           4064, MCAFEe1_Init},
```

最後に、src/ines.h ファイルをエディタで開いて、マッパーの初期化関数のプロトタイプ宣言を記述します。  
Finally, open the src/ines.h file in an editor and write the prototype declaration for the mapper initialization function.

```
void MCAFEe1_Init(CartInfo *);
```

