
#include "mapinc.h"

static const uint32 WRAMSIZE = 8*1024;

static uint8 *WRAM = NULL;
static uint8 IRQ_CNT;

static SFORMAT MCAFEe1_StateRegs[] =
{
	{ &IRQ_CNT, 1, "IRQC" },
	{ 0 }
};

DECLFW(MCAFEe1_MapperWrite) {
	switch (A & 0xf000) {
	case 0x8000:
		setchr4(0x0000, V & 0x1f);
		break;
	case 0x9000:
		setchr4(0x1000, V & 0x1f);
		break;
	case 0xa000:
		setprg8(0x8000, V & 0x0f);
		break;
	case 0xb000:
		setprg8(0xa000, V & 0x0f);
		break;
	case 0xe000:
		IRQ_CNT = V;
		X6502_IRQEnd(FCEU_IQEXT);
		break;
	case 0xf000:
		setmirror((V & 1) ? MI_H : MI_V);
		break;
	}
}

void MCAFEe1_HBlank(void) {
	if (IRQ_CNT > 0) {
		IRQ_CNT--;
		if (IRQ_CNT == 0) {
			X6502_IRQBegin(FCEU_IQEXT);
		}
	}
}

void MCAFEe1_Reset(void) {
	;
}

void MCAFEe1_Power(void) {

	// Register a handler for memory access.
	SetWriteHandler(0x8000, 0xffff, MCAFEe1_MapperWrite);
	SetReadHandler(0x8000, 0xffff, CartBR);
	SetWriteHandler(0x6000, 0x7fff, CartBW);
	SetReadHandler(0x6000, 0x7fff, CartBR);

	FCEU_CheatAddRAM(WRAMSIZE>>10, 0x6000, WRAM);

	// Bank settings for fixed area.
	setprg8r(0x10, 0x6000, 0);
	setprg8(0xc000, 0x0e);
	setprg8(0xe000, 0x0f);

	// initialize a inner registers.
	IRQ_CNT = 0;

	// Nametable milloring settings at startup.
	setmirror(MI_V);

	MCAFEe1_Reset();
}

void MCAFEe1_Close(void) {
	// Free extended memory.
	if (WRAM) FCEU_gfree(WRAM);
	WRAM = NULL;
}

void MCAFEe1_Init(CartInfo *info) {

	// Allocate extended memory.
	WRAM = (uint8*)FCEU_gmalloc(WRAMSIZE);
	SetupCartPRGMapping(0x10, WRAM, WRAMSIZE, 1);
	AddExState(WRAM, WRAMSIZE, 0, "WRAM");

	// Register memory for save.
	info->SaveGame[0] = WRAM;
	info->SaveGameLen[0] = WRAMSIZE;

	// Register internal register state
	AddExState(MCAFEe1_StateRegs, 0, 0, 0);

	// Register callback.
	info->Power = MCAFEe1_Power;
	info->Reset = MCAFEe1_Reset;
	info->Close = MCAFEe1_Close;

	// register HBlank hook.
	GameHBIRQHook = MCAFEe1_HBlank;
}

