// inesAlterer.cpp

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <ctype.h>

#define ALT_NUM     0x0001
#define ALT_PRG_RAM 0x0002
#define ALT_CHR_RAM 0x0004

typedef struct {
	char *fname;
	int alter;
	int type;
	int num;
	int sub;
	int prg_rom_size;
	int chr_rom_size;
	int prg_ram_size;
	int chr_ram_size;
	int prg_nvr_size;
	int chr_nvr_size;
	int mirroring;
	int save_memory;
	int tv_system;
	int bus_conflict;
} param_t;

int ParseArg(int argc, char *argv[], param_t *param);
int ShiftCount(int n);
int ParseHeader(uint8_t *header, param_t *param);
int BuildHeader(uint8_t *header, param_t *param);

int main(int argc, char *argv[]) {
	FILE *fp = NULL;
	param_t param;
	uint8_t header[16];
	int r;

	// 
	ParseArg(argc, argv, &param);

	// t@CJ
	if (param.fname == NULL) {
		goto bye;
	}

	fp = fopen(param.fname, "r+b");
	if (fp == NULL) {
		printf("File open error.\n");
		goto bye;
	}

	// ineswb_[ǂݍ
	fseek(fp, 0, SEEK_SET);
	fread(header, sizeof(uint8_t), 16, fp);
	r = ParseHeader(header, &param);
	if (r != 0) goto bye;

	// ineswb_[
	BuildHeader(header, &param);
	fseek(fp, 0, SEEK_SET);
	fwrite(header, sizeof(uint8_t), 16, fp);

bye:
	// t@C
	if (fp != NULL) fclose(fp);

	return 0;
}

int ParseArg(int argc, char *argv[], param_t *param) {
	char *ep;
	int phase = 0;
	int i, n;

	memset(param, 0, sizeof(param_t));
	param->num = -1;

	for (i = 0; i < argc; i++) {
		if (strcmp(argv[i], "-num") == 0) {
			i++;
			if (i < argc) {
				param->num = strtol(argv[i], NULL, 0);
				param->alter |= ALT_NUM;
			}
		} else if (strcmp(argv[i], "-prg_ram") == 0) {
			i++;
			if (i < argc) {
				n = strtol(argv[i], &ep, 0);
				if ((*ep == 'k') || (*ep == 'K')) n *= 1024;
				if ((*ep == 'm') || (*ep == 'M')) n *= 1024 * 1024;
				param->prg_ram_size = n;
				param->alter |= ALT_PRG_RAM;
			}
		} else if (strcmp(argv[i], "-chr_ram") == 0) {
			i++;
			if (i < argc) {
				n = strtol(argv[i], &ep, 0);
				if ((*ep == 'k') || (*ep == 'K')) n *= 1024;
				if ((*ep == 'm') || (*ep == 'M')) n *= 1024 * 1024;
				param->chr_ram_size = n;
				param->alter |= ALT_CHR_RAM;
			}
		} else {
			if (phase == 0) {
				param->fname = argv[i];
			}
		}
	}

	return 0;
}

int ShiftCount(int n) {
	int i = 15;
	while (i > 0) {
		if (n & (64<<i)) break;
		--i;
	}
	return i;
}

int ParseHeader(uint8_t *header, param_t *param) {
	int r = 0;

	param->type = (header[7] >> 2) & 3;
	if (param->type == 0) {
		param->prg_rom_size = header[4];
		param->chr_rom_size = header[5];
		if ((param->alter & ALT_PRG_RAM) == 0) {
			param->prg_ram_size = header[8] * 8 * 1024;
		}
		if ((param->alter & ALT_CHR_RAM) == 0) {
			param->chr_ram_size = (header[5] == 0) ? 8*1024 : 0;
		}
		param->prg_nvr_size = 0;
		param->chr_nvr_size = 0;
		if ((param->alter & ALT_NUM) == 0) {
			param->num = ((header[7] & 0xf0)   ) |
			             ((header[6] & 0xf0)>>4);
		}
		param->sub          = 0;
		param->mirroring    = ((header[6]>>2) & 2) | ((header[6]   ) & 1);
		param->save_memory  = ((header[6]>>1) & 1);
		param->tv_system    = ((header[10]<<1) & 2) | ((header[10]>>1) & 1);
		param->bus_conflict = ((header[10]>>5) & 1);
	} else if (param->type == 2) {
		param->prg_rom_size = (((header[9]   )&15)<<8)|header[4];
		param->chr_rom_size = (((header[9]>>4)&15)<<8)|header[5];
		if ((param->alter & ALT_PRG_RAM) == 0) {
			param->prg_ram_size = (((header[10]   )&15) == 0) ? 0 : 64 << ((header[10]   )&15);
		}
		if ((param->alter & ALT_CHR_RAM) == 0) {
			param->chr_ram_size = (((header[11]   )&15) == 0) ? 0 : 64 << ((header[11]   )&15);
		}
		param->prg_nvr_size = (((header[10]>>4)&15) == 0) ? 0 : 64 << ((header[10]>>4)&15);
		param->chr_nvr_size = (((header[11]>>4)&15) == 0) ? 0 : 64 << ((header[11]>>4)&15);
		if ((param->alter & ALT_NUM) == 0) {
			param->num = ((header[8] & 0x0f)<<8) |
			             ((header[7] & 0xf0)   ) |
			             ((header[6] & 0xf0)>>4);
		}
		param->sub          = ((header[8]>>4) & 15);;
		param->mirroring    = ((header[6]>>2) & 2) | ((header[6]   ) & 1);
		param->save_memory  = ((header[6]>>1) & 1);
		param->tv_system    = ((header[12]  ) & 3);
		param->bus_conflict = 0;
	} else {
		r = 1;
	}

	return r;
}

int BuildHeader(uint8_t *header, param_t *param) {
	int r = 0;

	if (param->num >= 256) param->type = 2;

	if (param->type == 0) {
		header[ 4] = param->prg_rom_size;
		header[ 5] = param->chr_rom_size;
		header[ 6] = (header[6] & 0x04) |
		             (((param->num           )&15)<<4) |
		             (((param->mirroring  >>1)& 1)<<3) |
		             (((param->save_memory   )& 1)<<1) |
		             (((param->mirroring     )& 1)<<3);
		header[ 7] = (header[7] & 0x03) |
		             (((param->num        >>4)&15)<<4);
		header[ 8] = param->prg_ram_size / (8 * 1024);
		header[ 9] = (param->tv_system == 1) ? 1 : 0;
		header[10] = (((param->bus_conflict)&1)<<5) |
		             ((param->prg_ram_size == 0) ? 0x10 : 0) |
		             (((param->tv_system   )&1)<<1) |
		             (((param->tv_system>>1)&1)   );
	} else if (param->type == 2) {
		header[ 4] = param->prg_rom_size & 0xff;
		header[ 5] = param->chr_rom_size & 0xff;
		header[ 6] = (header[6] & 0x04) |
		             (((param->num           )&15)<<4) |
		             (((param->mirroring  >>1)& 1)<<3) |
		             (((param->save_memory   )& 1)<<1) |
		             (((param->mirroring     )& 1)<<3);
		header[ 7] = (header[7] & 0x03) |
		             0x08 |
		             (((param->num        >>4)&15)<<4);
		header[ 8] = (((param->sub   )&15)<<4) |
		             (((param->num>>8)&15)   );
		header[ 9] = (((param->chr_rom_size>>8)&15)<<4) |
		             (((param->prg_rom_size>>8)&15)   );
		header[10] = (ShiftCount(param->prg_nvr_size)<<4) |
		             (ShiftCount(param->prg_ram_size)   );
		header[11] = (ShiftCount(param->chr_nvr_size)<<4) |
		             (ShiftCount(param->chr_ram_size)   );
		header[12] = (param->tv_system & 3);
	} else {
		r = 1;
	}

	return r;
}

