; User task (main)

Irq_Func:
	.dw	User_Irq_Bank1
	.dw	User_Irq_Bank1
	.dw	User_Irq_Bank1
	.dw	User_Irq_Bank2

User:
	; screen off
	lda	#$00
	sta	$2001

	; milloring setting
	lda	#$01	; H milloring
	sta	$f000

	; bank settings
	lda	#12
	sta	$a000
	lda	#13
	sta	$b000
	lda	#0
	sta	$8000
	lda	#1
	sta	$9000

	; palette initilize
	ldx	#PS_BG_TEST
	jsr	Pal_SetBGPalette
	ldx	#PS_SP_TEST
	jsr	Pal_SetSPPalette
	lda	#$40
	jsr	Vcmd_SetPalette

	; scroll settings
	lda	#$00
	sta	<SCROLL_X
	sta	<SCROLL_Y

	; IRQ settings
;	lda	$00
	sta	$e000
	sta	<IRQ_LINE

	jsr	Yeild		; wait a next frame

	; set jmp instruction
	lda	#$6c
	sta	<USR_FNC-1
	sta	IRQ_JMP
	lda	#LOW(Irq_Func)
	sta	IRQ_JMP+1
	lda	#HIGH(Irq_Func)
	sta	IRQ_JMP+2
	lda	#$00
	sta	<USR_FNC
	sta	<USR_FNC+1

	; NMI task settings
	lda	#LOW(User_Nmi)
	sta	NMI_FNC
	lda	#HIGH(User_Nmi)
	sta	NMI_FNC+1

	; sound settings
	lda	#LOW(Sound_PlayPCM)
	ldx	#HIGH(Sound_PlayPCM)
	jsr	Fssq_SetDpcmCB

	; sprite initialize
	jsr	Spr_Init

	jsr	Yeild_SetEntry	; next frame resumes from here

	; clear screen
	lda	#LOW(Vcmd_Clear)
	ldx	#HIGH(Vcmd_Clear)
	jsr	Vcmd_SequenceData

	jsr	Yeild_Eq	; wait a next frame

	; screen on
	lda	#$88
	sta	<PPU_REG1
	sta	$2000
	lda	#$1e
	sta	$2001

	jsr	Yeild		; wait a next frame

Test_Init:
	jsr	Spr_Init

	; draw a test screen
	lda	#LOW(Vcmd_Frame)
	ldx	#HIGH(Vcmd_Frame)
	jsr	Vcmd_SequenceData

	jsr	Yeild_Eq	; wait a next frame

	jsr	FadeIn


Test_Main:
	jsr	TestPrgBank0
	ldx	#$00
	jsr	Draw_Result

	jsr	Yeild		; wait a next frame

	jsr	TestPrgBank1
	ldx	#$01
	jsr	Draw_Result

	jsr	Yeild		; wait a next frame

	jsr	TestChrBank0
	ldx	#$02
	jsr	Draw_Result

	jsr	Yeild		; wait a next frame

	jsr	TestChrBank1
	ldx	#$03
	jsr	Draw_Result

	jsr	Yeild		; wait a next frame

	jsr	TestBBackup1
	ldx	#$04
	jsr	Draw_Result

	jsr	Yeild		; wait a next frame

	jsr	TestWorkRam1
	ldx	#$05
	jsr	Draw_Result

	jsr	Yeild		; wait a next frame

Test_Wait:
	lda	#0
	sta	<USR_CNT

	jsr	Yeild_SetEntry	; next frame resumes from here

	; initialize sprite
	jsr	Spr_Init

	dec	<USR_CNT

	jsr	Yeild_Eq	; wait a next frame

	jsr	FadeOut

	jsr	Yeild_Eq	; wait a next frame

Demo_Init:
	lda	#BANK_SNAP1
	sta	<BANK_CUR
	sta	<BANK_START
	lda	#BANK_SNAP1+8
	sta	<BANK_END

	; IRQ task settings
	ldx	#LOW(IRQ_JMP)
	ldy	#HIGH(IRQ_JMP)
	stx	IRQ_FNC
	sty	IRQ_FNC+1
	lda	#8
	sta	<IRQ_LINE

	; initialize palette
	ldx	#PS_BG_TEST+1
	jsr	Pal_SetBGPalette
	ldx	#PS_SP_TEST+1
	jsr	Pal_SetSPPalette

	; play bgm
	lda	#$80+BANK_PRG_BGM
	jsr	Fssq_SetBankBGM
	lda	#LOW(Bgm1_PlayData)
	ldx	#HIGH(Bgm1_PlayData)
	jsr	Fssq_PlayBGM

	; draw snap
	lda	#LOW(Vcmd_Snap)
	ldx	#HIGH(Vcmd_Snap)
	jsr	Vcmd_SequenceData

	jsr	Yeild_Eq	; wait a next frame

	jsr	FadeIn

	jsr	Yeild_Eq	; wait a next frame

Loop:
	jsr	Yeild_SetEntry	; next frame resumes from here

	; initialize sprite
	jsr	Spr_Init

	jsr	Yeild_ReEntry


TestPrgBank0:
	ldx	#0
	lda	#14
	sta	<FNC_WORK
.1:
	lda	<FNC_WORK
	sta	$a000
	lda	$9fff
	cmp	<FNC_WORK
	beq	.2
	inx
.2:
	dec	<FNC_WORK
	bpl	.1
	txa
	rts

TestPrgBank1:
	ldx	#0
	lda	#14
	sta	<FNC_WORK
.1:
	lda	<FNC_WORK
	sta	$b000
	lda	$bfff
	cmp	<FNC_WORK
	beq	.2
	inx
.2:
	dec	<FNC_WORK
	bpl	.1
	txa
	rts

TestChrBank0:
	ldx	#$00
	stx	$2001
	lda	#31
	sta	<FNC_WORK
.1:
	lda	<FNC_WORK
	sta	$8000

	lda	#$0f
	sta	$2006
	lda	#$ff
	sta	$2006

	lda	$2007
	lda	$2007
	cmp	<FNC_WORK
	beq	.2
	inx
.2:
	dec	<FNC_WORK
	bpl	.1

	lda	#0
	sta	$8000
	lda	#$1e
	sta	$2001

	txa
	rts

TestChrBank1:
	ldx	#$00
	stx	$2001
	lda	#31
	sta	<FNC_WORK
.1:
	lda	<FNC_WORK
	sta	$9000

	lda	#$1f
	sta	$2006
	lda	#$ff
	sta	$2006

	lda	$2007
	lda	$2007
	cmp	<FNC_WORK
	beq	.2
	inx
.2:
	dec	<FNC_WORK
	bpl	.1

	lda	#1
	sta	$9000
	lda	#$1e
	sta	$2001

	txa
	rts

TestWorkRam1:
	lda	#$55
	ldy	#$55
	jsr	Util_SetLFSR

	lda	#$60
	sta	<FNC_WORK+1
	ldy	#$00
	sty	<FNC_WORK
.1:
	sty	<FNC_WORK+3
	jsr	Util_UpdateLFSR
	sty	<FNC_WORK+2
	ldy	<FNC_WORK+3
	sta	[FNC_WORK],y
	iny
	lda	<FNC_WORK+2
	sta	[FNC_WORK],y

	iny
	bne	.1
	inc	<FNC_WORK+1
	lda	<FNC_WORK+1
	cmp	#$80
	bne	.1

TestBBackup1:
	lda	#$55
	ldy	#$55
	jsr	Util_SetLFSR

	lda	#$60
	sta	<FNC_WORK+1
	ldy	#$00
	sty	<FNC_WORK
	sty	<FNC_WORK+4
.2:
	sty	<FNC_WORK+3
	jsr	Util_UpdateLFSR
	sty	<FNC_WORK+2
	ldy	<FNC_WORK+3
	cmp	[FNC_WORK],y
	beq	.3
	sec
	rol	<FNC_WORK+4
.3:
	iny
	lda	<FNC_WORK+2
	cmp	[FNC_WORK],y
	beq	.4
	sec
	rol	<FNC_WORK+4
.4:
	iny
	bne	.2
	inc	<FNC_WORK+1
	lda	<FNC_WORK+1
	cmp	#$80
	bne	.2

	lda	<FNC_WORK+4
	rts

	; fadein a screen
FadeIn:
	lda	#$07
	sta	<USR_CNT
	jsr	Yeild_SetEntry	; next frame resumes from here

	lda	<USR_CNT
	asl	a
	asl	a
	asl	a
	and	#$f0
	jsr	Vcmd_SetPalette

	dec	<USR_CNT
	bne	.1
	jmp	Yeild		; wait a next frame
.1:
	jmp	_ReEntry

	; fadeout a screen
FadeOut:
	lda	#$01
	sta	<USR_CNT
	jsr	Yeild_SetEntry	; next frame resumes from here

	lda	<USR_CNT
	asl	a
	asl	a
	asl	a
	and	#$f0
	jsr	Vcmd_SetPalette

	inc	<USR_CNT
	lda	<USR_CNT
	cmp	#$09
	bne	.1
	jmp	Yeild		; wait a next frame
.1:
	jmp	_ReEntry

	; a: 0:display OK other:display NG
Draw_Result:
	cmp	#$01
	txa
	rol	a
	tay
	lda	.lo,y
	ldx	.hi,y
	jmp	Vcmd_SequenceData
.lo:
	.db	LOW(Vcmd_Result1_OK)
	.db	LOW(Vcmd_Result1_NG)
	.db	LOW(Vcmd_Result2_OK)
	.db	LOW(Vcmd_Result2_NG)
	.db	LOW(Vcmd_Result3_OK)
	.db	LOW(Vcmd_Result3_NG)
	.db	LOW(Vcmd_Result4_OK)
	.db	LOW(Vcmd_Result4_NG)
	.db	LOW(Vcmd_Result5_OK)
	.db	LOW(Vcmd_Result5_NG)
	.db	LOW(Vcmd_Result6_OK)
	.db	LOW(Vcmd_Result6_NG)
.hi:
	.db	HIGH(Vcmd_Result1_OK)
	.db	HIGH(Vcmd_Result1_NG)
	.db	HIGH(Vcmd_Result2_OK)
	.db	HIGH(Vcmd_Result2_NG)
	.db	HIGH(Vcmd_Result3_OK)
	.db	HIGH(Vcmd_Result3_NG)
	.db	HIGH(Vcmd_Result4_OK)
	.db	HIGH(Vcmd_Result4_NG)
	.db	HIGH(Vcmd_Result5_OK)
	.db	HIGH(Vcmd_Result5_NG)
	.db	HIGH(Vcmd_Result6_OK)
	.db	HIGH(Vcmd_Result6_NG)

Sound_PlayPCM:
	lda	#$0f
	sta	$4010
	lda	#$40
	sta	$4011
	lda	#((PcmKick-$c000)/64)
	sta	$4012
	lda	#((PcmKick_End-PcmKick-1)/16)
	sta	$4013

	lda	$4015
	and	#$0f
	sta	$4015
	ora	#$10
	sta	$4015
	rts

User_Nmi:
	lda	<PPU_REG1
	and	#$fe
	sta	<PPU_REG1

	lda	SYN_BANK
	sta	$8000
	lda	SYN_BANK+1
	sta	$9000

	lda	$2002	; clear address latch

	rts

				; 26
User_Irq_Bank1:
	lda	#56		; 2
	inc	IRQ_JMP+1	; 6
	inc	IRQ_JMP+1	; 6
	sta	$e000		; 4

	ldy	#10		; 2
.1:
	dey			; 2
	bpl	.1		; 2/3
	nop			; 2

	ldy	<BANK_CUR	; 3
	sty	$8000		; 4
	iny
	sty	<BANK_CUR
	rts

User_Irq_Bank2:
	lda	#0		; 2
	sta	$e000		; 4
	sta	IRQ_JMP+1	; 4

	ldy	#12		; 2
.1:
	dey			; 2
	bpl	.1		; 2/3
	nop			; 2

	ldy	<BANK_CUR	; 3
	sty	$8000		; 4
	ldy	<BANK_START
	sty	<BANK_CUR
	rts

; subroutines (palette)

	; a : 0000nnnn n:scene num
	; use reg : a x y
Pal_SetBGPalette:
	lda	Xcn,x
	ora	#$0f
	tax
	ldy	#15
.1:
	lda	Pal_Data_Frame,x
	sta	PAL_BGBUF,y
	dex
	dey
	bpl	.1
	rts

	; a : 0000nnnn n:scene num
	; use reg : a x y
Pal_SetSPPalette:
	lda	Xcn,x
	ora	#$0f
	tax
	ldy	#15
.1:
	lda	Pal_Data_Object,x
	sta	PAL_BGBUF+16,y
	dex
	dey
	bpl	.1
	rts


; subroutines (sprite)

Spr_Init:
	ldx	#$00
.1:
	lda	#240
	sta	SPR_DATA,x
	lda	Int+4,x
	tax
	bne	.1

	lda	#1
	sta	<SPR_UPD
	rts

; subroutines (Vcmd)

	; a : 0ddd0ttt d:darkness value 
	;     t:target (0:BG&SP 1:BG    2:SP       3:
	;               4:BG0   5:BG1-3 6:BG1-3&SP 7:)
	; use reg : a x y
	; use work : 0-1
Vcmd_SetPalette:
	and	#$77
	sta	<FNC_WORK
	and	#$07
	tax
	eor	<FNC_WORK
	sta	<FNC_WORK
	lda	.size,x
	sta	<FNC_WORK+1
	ldy	.offset,x
.1:
	lda	PAL_BGBUF,y
	sec
	sbc	<FNC_WORK
	bmi	.3
	cmp	#$0d
	bne	.4
.3:
	lda	#$1d
.4:
	sta	PAL_BGBUF2,y
	iny
	dec	<FNC_WORK+1
	bne	.1

	ldx	<VCMD_SIZE
	lda	#32
	sta	VCMD_DATA,x	; size
	lda	#$00
	sta	VCMD_DATA+1,x	; attr
	sta	VCMD_DATA+3,x	; ppu addr high
	lda	#$3f
	sta	VCMD_DATA+2,x	; ppu addr high
	lda	Int+36,x
	sta	<VCMD_SIZE
	tax

	ldy	#31
.5:
	dex
	lda	PAL_BGBUF2,y
	sta	VCMD_DATA,x
	dey
	bpl	.5

	rts
.offset:
	.db	 0,  0, 16,  0,  0,  4,  4,  0
.size:
	.db	32, 16, 16, 32,  4, 12, 28, 32

	; a : value (lo)
	; y : value (hi)
Util_SetLFSR:
	sta	<LFSR
	sty	<LFSR+1
	rts

	; a : result (lo)
	; y : result (hi)
Util_UpdateLFSR:
	lda	<LFSR
	bit	Int+$04
	beq	.1
	eor	#$01
.1:
	bit	Int+$08
	beq	.2
	eor	#$01
.2:
	bit	Int+$20
	beq	.3
	eor	#$01
.3:
	lsr	a
	ror	<LFSR+1
	ror	<LFSR

	lda	<LFSR
	ldy	<LFSR+1
	rts

DataSection:

; palettes

Pal_Data_Frame:
	.db	$1d, $26, $37, $20	; test
	.db	$1d, $1d, $1d, $1d
	.db	$1d, $1d, $1d, $1d
	.db	$1d, $1d, $1d, $1d
	.db	$07, $17, $27, $37	; snap
	.db	$07, $1d, $1d, $1d
	.db	$07, $1d, $1d, $1d
	.db	$07, $1d, $1d, $1d
Pal_Data_Object:
	.db	$1d, $00, $10, $20	; test
	.db	$1d, $1d, $1d, $1d
	.db	$1d, $1d, $1d, $1d
	.db	$1d, $1d, $1d, $1d
	.db	$07, $00, $10, $20	; snap
	.db	$07, $1d, $1d, $1d
	.db	$07, $1d, $1d, $1d
	.db	$07, $1d, $1d, $1d

; nametables

Clear_Data1:
	.db	VCX(0,1), BG_TILE, VCX(1,63)
Clear_Data1_End1:
	.db	VCX(1,64)
Clear_Data1_End2:
Clear_Data2:
	.db	VCX(0,1), $00, VCX(1,7)
Clear_Data2_End:
Vcmd_Clear:
	VCMD_SET	$00, $2380, Clear_Data1, Clear_Data1_End1
	VCMD_SET	$80, $23f8, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2780, Clear_Data1, Clear_Data1_End1
	VCMD_SET	$80, $27f8, Clear_Data2, Clear_Data2_End

	VCMD_SET	$00, $2000, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $23c0, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2080, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $23c8, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2100, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $23d0, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2180, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $23d8, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2200, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $23e0, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2280, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $23e8, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2300, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $23f0, Clear_Data2, Clear_Data2_End

	VCMD_SET	$00, $2400, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $27c0, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2480, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $27c8, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2500, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $27d0, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2580, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $27d8, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2600, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $27e0, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2680, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$80, $27e8, Clear_Data2, Clear_Data2_End
	VCMD_SET	$00, $2700, Clear_Data1, Clear_Data1_End2
	VCMD_SET	$c0, $27f0, Clear_Data2, Clear_Data2_End

; ABCGHKMNOPRUW.
Frame_Data1:
	.db	VCX(0,9), BG_CH+19, BG_CH+20, BG_CH+13
	.db	          BG_TILE,  BG_CH+11, BG_CH+10
	.db	          BG_CH+17, BG_CH+15, BG_CH+0
Frame_Data1_End:
Frame_Data2:
	.db	VCX(0,9), BG_CH+19, BG_CH+20, BG_CH+13
	.db	          BG_TILE,  BG_CH+11, BG_CH+10
	.db	          BG_CH+17, BG_CH+15, BG_CH+1
Frame_Data2_End:
Frame_Data3:
	.db	VCX(0,9), BG_CH+12, BG_CH+14, BG_CH+20
	.db	          BG_TILE,  BG_CH+11, BG_CH+10
	.db	          BG_CH+17, BG_CH+15, BG_CH+0
Frame_Data3_End:
Frame_Data4:
	.db	VCX(0,9), BG_CH+12, BG_CH+14, BG_CH+20
	.db	          BG_TILE,  BG_CH+11, BG_CH+10
	.db	          BG_CH+17, BG_CH+15, BG_CH+1
Frame_Data4_End:
Frame_Data5:
	.db	VCX(0,8), BG_CH+11, BG_CH+23, BG_CH+11
	.db	          BG_CH+10, BG_CH+12, BG_CH+15
	.db	          BG_CH+21, BG_CH+19
Frame_Data5_End:
Frame_Data6:
	.db	VCX(0,4), BG_CH+22, BG_CH+20, BG_CH+10
	.db	          BG_CH+16
Frame_Data6_End:
Vcmd_Frame:
	VCMD_SET	$00, $2084, Frame_Data1,  Frame_Data1_End
	VCMD_SET	$00, $20c4, Frame_Data2,  Frame_Data2_End
	VCMD_SET	$80, $2104, Frame_Data3,  Frame_Data3_End
	VCMD_SET	$00, $2144, Frame_Data4,  Frame_Data4_End
	VCMD_SET	$00, $2184, Frame_Data5,  Frame_Data5_End
	VCMD_SET	$c0, $21c4, Frame_Data6,  Frame_Data6_End

Result_Data1:
	.db	VCX(0,2), BG_CH+18, BG_CH+15
Result_Data1_End:
Result_Data2:
	.db	VCX(0,2), BG_CH+17, BG_CH+13
Result_Data2_End:
Vcmd_Result1_OK:
	VCMD_SET	$c0, $2090, Result_Data1, Result_Data1_End
Vcmd_Result1_NG:
	VCMD_SET	$c0, $2090, Result_Data2, Result_Data2_End
Vcmd_Result2_OK:
	VCMD_SET	$c0, $20d0, Result_Data1, Result_Data1_End
Vcmd_Result2_NG:
	VCMD_SET	$c0, $20d0, Result_Data2, Result_Data2_End
Vcmd_Result3_OK:
	VCMD_SET	$c0, $2110, Result_Data1, Result_Data1_End
Vcmd_Result3_NG:
	VCMD_SET	$c0, $2110, Result_Data2, Result_Data2_End
Vcmd_Result4_OK:
	VCMD_SET	$c0, $2150, Result_Data1, Result_Data1_End
Vcmd_Result4_NG:
	VCMD_SET	$c0, $2150, Result_Data2, Result_Data2_End
Vcmd_Result5_OK:
	VCMD_SET	$c0, $2190, Result_Data1, Result_Data1_End
Vcmd_Result5_NG:
	VCMD_SET	$c0, $2190, Result_Data2, Result_Data2_End
Vcmd_Result6_OK:
	VCMD_SET	$c0, $21d0, Result_Data1, Result_Data1_End
Vcmd_Result6_NG:
	VCMD_SET	$c0, $21d0, Result_Data2, Result_Data2_End

Snap_Data1:
	.db	VCX(0,32), $04, $05, $06, $07, $08, $09, $0a, $0b
	.db	           $0c, $0d, $0e, $0f, $10, $11, $12, $13
	.db	           $14, $15, $16, $17, $18, $19, $1a, $1b
	.db	           $1c, $1d, $1e, $1f, $20, $21, $22, $23
	.db	VCX(0,64), $24, $25, $26, $27, $28, $29, $2a, $2b
	.db	           $2c, $2d, $2e, $2f, $30, $31, $32, $33
	.db	           $34, $35, $36, $37, $38, $39, $3a, $3b
	.db	           $3c, $3d, $3e, $3f, $40, $41, $42, $43
	.db	           $44, $45, $46, $47, $48, $49, $4a, $4b
	.db	           $4c, $4d, $4e, $4f, $50, $51, $52, $53
	.db	           $54, $55, $56, $57, $58, $59, $5a, $5b
	.db	           $5c, $5d, $5e, $5f, $60, $61, $62, $63
Snap_Data1_End:
Snap_Data2:
	.db	VCX(0,64), $64, $65, $66, $67, $68, $69, $6a, $6b
	.db	           $6c, $6d, $6e, $6f, $70, $71, $72, $73
	.db	           $74, $75, $76, $77, $78, $79, $7a, $7b
	.db	           $7c, $7d, $7e, $7f, $80, $81, $82, $83
	.db	           $84, $85, $86, $87, $88, $89, $8a, $8b
	.db	           $8c, $8d, $8e, $8f, $90, $91, $92, $93
	.db	           $94, $95, $96, $97, $98, $99, $9a, $9b
	.db	           $9c, $9d, $9e, $9f, $a0, $a1, $a2, $a3
	.db	VCX(0,64), $a4, $a5, $a6, $a7, $a8, $a9, $aa, $ab
	.db	           $ac, $ad, $ae, $af, $b0, $b1, $b2, $b3
	.db	           $b4, $b5, $b6, $b7, $b8, $b9, $ba, $bb
	.db	           $bc, $bd, $be, $bf, $c0, $c1, $c2, $c3
	.db	           $c4, $c5, $c6, $c7, $c8, $c9, $ca, $cb
	.db	           $cc, $cd, $ce, $cf, $d0, $d1, $d2, $d3
	.db	           $d4, $d5, $d6, $d7, $d8, $d9, $da, $db
	.db	           $dc, $dd, $de, $df, $e0, $e1, $e2, $e3
Snap_Data2_End:
Vcmd_Snap:
	VCMD_SET	$80, $2020, Snap_Data1, Snap_Data1_End
	VCMD_SET	$80, $2080, Snap_Data2, Snap_Data2_End
	VCMD_SET	$80, $2100, Snap_Data1, Snap_Data1_End
	VCMD_SET	$80, $2160, Snap_Data2, Snap_Data2_End
	VCMD_SET	$80, $21e0, Snap_Data1, Snap_Data1_End
	VCMD_SET	$80, $2240, Snap_Data2, Snap_Data2_End
	VCMD_SET	$80, $22c0, Snap_Data1, Snap_Data1_End
	VCMD_SET	$c0, $2320, Snap_Data2, Snap_Data2_End

